==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ron/db/dbrecordfield.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;


use Ron\DateUtils;
use Ron\NumberUtils;

class DBRecordField
{
    const DEFAULT_MAX_LOG_LENGTH = 1024;

    /** @var DBRecord */
    var $record;

    /** @var string */
    var $fieldName;
    /** @var string */
    var $format;
    /** @var int    Delka textoveho pole */
    var $size;

    /** @var bool   Priznak, ze se nema updatovat */
    var $readOnly;

    /** @var bool   Priznak, ze se muze menit nenulova hodnota. Pokud je false, tak nelze menit jednou zadanou hodnotu. */
    var $changeable = true;

    /** @var bool   Priznak, ze je pole jen pro logovani updatu (napr. Update_UserName) - hodnota = <strong>true</strong> <br />
    zmena hodnoty tohoto pole "nevynuti" UPDATE */
    var $updateLog;

    /** @var int    Maximalni delka logovane hodnoty: -1=neomezeno, 0=nelogovat, >0 pocet znaku */
    var $maxLogLength;

    /** @var bool   V insertu se nevyplnuje */
    var $noInsert;

    /** @var mixed  Hodnota nactena z databaze */
    var $originalValue;
    /** @var mixed  Hodnota zadana uzivatelem */
    var $value;


    function __construct($name, $format, $readOnly = false, $size = 0)
    {
        $this ->fieldName = $name;
        $this ->format = $format;
        $this ->size = (int)$size;

        $this ->readOnly = $readOnly;

        $this ->updateLog = false;
        $this ->noInsert = false;

        $this ->maxLogLength = self::DEFAULT_MAX_LOG_LENGTH;
    }


    function setValue($value)
    {
        if ($this ->size > 0 && isset($value) && $value != '') $value = substr($value, 0, $this ->size);

        // Nastavi hodnotu promenne
        $this ->value = $value;

        return $this;
    }


    function setNewValue($value)
    {
        $this ->setValue($value);
        $this ->readOnly = false;
        return $this;
    }

    function setUpdateLog($value)
    {
        $this ->updateLog = $value;
        return $this;
    }

    /**
     * @param boolean $noInsert
     * @return DBRecordField
     */
    public function setNoInsert($noInsert)
    {
        $this->noInsert = $noInsert;
        return $this;
    }

    /**
     * @param boolean $readOnly
     * @return DBRecordField
     */
    public function setReadOnly($readOnly)
    {
        $this->readOnly = $readOnly;
        return $this;
    }

    /**
     * @param boolean $changeable
     * @return DBRecordField
     */
    public function setChangeable($changeable)
    {
        $this->changeable = $changeable;
        return $this;
    }





    /**
     * Vraci hodnotu pro zapis do SQL dotazu
     * @return string
     */
    function getSQLValue()
    {
        if (!isset($this ->value) || $this ->value === '') return 'null';

        switch ($this ->format) {
            case 'int' :
            case 'integer' :
                return (int)$this ->value;

            case 'float' :
            case 'double' :
                return (float)$this ->value;

            case 'date' :
                return $this ->record ->database ->dateToString($this ->value);

            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->timestampToString($this ->value);

            case 'sqltimestamp' : // hodnota zadana ve value je nacteny timestamp z databaze (pouziva se v XML)
                return $this ->record ->database ->timestampToString($this ->record ->database ->stringToTimestamp($this ->value));

            case 'sqldate' :
                return $this ->record ->database ->dateToString($this ->record ->database ->stringToTimestamp($this ->value));

            case 'datestr' :
                return $this ->record ->database ->timestampToString(DateUtils::dateStringToTimestamp($this ->value));

            case 'time' :
                return $this ->record ->database ->timeToString($this ->value);

            case 'memo' :
            case 'blob' :
            case 'binary' : // stejne jako blob, ale neloguje se hodnota
            default :
                return "'" . $this ->record ->database ->addSlashes($this ->value) . "'";
        }
    }


    /**
     * Vraci hodnotu pro zapis do SQL parametru
     * @return string
     */
    function getSQLParam()
    {
        if ($this ->format == 'blob'  ||  $this ->format == 'binary') {
            // pro sqlsrv je potreba i null hodnotu nebo prazdny retezec prohnat pres getBlobParam!
            return $this ->record ->database ->getBlobParam($this ->value);
        }

        if (!isset($this ->value) || $this ->value === '') return null;

        switch ($this ->format) {
            case 'int' :
            case 'integer' :
                return (int)$this ->value;

            case 'float' :
            case 'double' :
                return (float)$this ->value;

            case 'date' :
                return $this ->record ->database ->dateToParam($this ->value);

            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->timestampToParam($this ->value);

            case 'sqltimestamp' : // hodnota zadana ve value je nacteny timestamp z databaze (pouziva se v XML)
                return $this ->record ->database ->timestampToParam($this ->record ->database ->stringToTimestamp($this ->value));

            case 'sqldate' :
                return $this ->record ->database ->dateToParam($this ->record ->database ->stringToTimestamp($this ->value));

            case 'datestr' :
                return $this ->record ->database ->timestampToParam(DateUtils::dateStringToTimestamp($this ->value));

            case 'time' :
                return $this ->record ->database ->timeToParam($this ->value);

            case 'memo' :
            default :
                return $this ->value;
        }
    }


    /**
     * Prevede sql hodnotu na datovou hodnotu (int/float/...)
     * @param string $sqlvalue
     * @return mixed
     */
    function getValueFromSQLData($sqlvalue)
    {
        if (!isset($sqlvalue)) return null;

        switch ($this ->format) {
            case 'date' :
            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->stringToTimestamp($sqlvalue);

            case 'time' :
                //return $this ->record ->database ->stringToTimestamp($sqlvalue);
                return $this ->record ->database ->stringToSecondsOfDay($sqlvalue);

            case 'float' :
                return (float)$sqlvalue;

            case 'int' :
                return (int)$sqlvalue;

            case 'datestr' :
                return $sqlvalue; // tohle by nemelo nastat

            case 'blob' :
            case 'binary' :
                return $this ->record ->database ->stringToBinary($sqlvalue);

            //case 'sqltimestamp' : - neprevadi, zanecha puvodni DB format
            //case 'sqldate' : - neprevadi
        }

        return $sqlvalue;
    }


    /**
     * Naformatuje hodnotu pro textove zobrazeni - napr. do logu
     * @param $value
     * @return bool|int|null|string
     */
    function formatValue($value)
    {
        if (!isset($value) || $value === '') return '';

        switch ($this ->format) {
            /** @noinspection PhpMissingBreakStatementInspection */
            case 'sqldate' :
                $value = $this ->record ->database ->stringToTimestamp($value); // value je v databazovem formatu - nejdrive se musi prevest
            // pokracuje dale!
            case 'date' :
                return date('j.n.Y', (int)$value);

            /** @noinspection PhpMissingBreakStatementInspection */
            case 'sqltimestamp' :
                $value = $this ->record ->database ->stringToTimestamp($value);
            // pokracuje dale!
            case 'datetime' :
            case 'datestr' :
            case 'timestamp' :
                return date('j.n.Y G:i:s', (int)$value);

            case 'time' :
                return DateUtils::secondsToString($value); //date('j.n.Y G:i:s', (int)$value);


            case 'binary' :
                return '(' . strlen($this ->value) . 'B' . ")";
            case 'blob' :
                return '(' . strlen($this ->value) . 'B' . ")\n" . $value;
        }

        return $value;
    }


    function isChanged()
    {
        if (!$this ->changeable  &&  isset($this ->originalValue)) {
            return false;
        }

        if ($this ->format == 'float') {
            if (isset($this ->originalValue) && !isset($this ->value)) return true;
            if (isset($this ->value) && !isset($this ->originalValue)) return true;

            if (NumberUtils::floatsEqual($this ->originalValue, $this ->value)) return false;

            return true;
        }

        if (isset($this ->originalValue)  &&  ($this ->originalValue !== '')  && !isset($this ->value)) return true;
        if (!isset($this ->originalValue)  &&  isset($this ->value)) {
            return $this ->value !== '';
        }

        if ($this ->format == 'string'  ||  $this ->format == 'memo'  ||  $this ->format == '') {
            // pro retezcova pole kontroluje presnou shodu, aby fungovala i zmena napr. z 0068 na 68
            return $this ->originalValue !== $this ->value;
        }

        return $this ->originalValue != $this ->value;
    }


    function getLogValue()
    {
        if (!$this ->maxLogLength) return ''; // 0 = nelogovat

//        $value = $this ->readOnly
//            ? $this ->originalValue
//            : (($this ->changeable  ||  !isset($this ->originalValue)) ? $this ->value : $this ->originalValue);

        $value = $this ->value;

        if (!isset($value)) return '(NULL)';

        // TODO: umoznit logovat jen poslednich X znaku? napr. pokud bude zadano -100, tak bude logovat jen poslednich 100 znaku.
        // (muze se hodit napr. pro HistorieSchvalovani)
        $log = $this ->formatValue($value);
        if ($this ->maxLogLength > 0) $log = substr($log, 0, $this ->maxLogLength);

        return $log;
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ron/db/dbrecordfield.php <==
<?php
/**
 * stanislav.elbl, 19.02.2021
 */

namespace Ron\DB;


use Ron\DateUtils;
use Ron\NumberUtils;
use Ron\StringUtils;
use Ron\TextManager;

class DBRecordField
{
    const DEFAULT_MAX_LOG_LENGTH = 1024;

    /** @var DBRecord */
    public $record;

    /** @var string */
    public $fieldName;
    /** @var string */
    public $format;
    /** @var int    Delka textoveho pole */
    public $size;

    /** @var string Jmeno sekvence pro pole formatu "sequence". Pokud neni zadano, pouzije se SEQ_<jmenotabulky> */
    public $sequenceName;

    /** @var bool   Priznak, ze se nema updatovat */
    public $readOnly;

    /** @var bool   Priznak, ze se muze menit nenulova hodnota. Pokud je false, tak nelze menit jednou zadanou hodnotu. */
    public $changeable = true;

    /** @var bool   Priznak, ze je pole jen pro logovani updatu (napr. Update_UserName) - hodnota = <strong>true</strong> <br />
    zmena hodnoty tohoto pole "nevynuti" UPDATE */
    public $updateLog;

    /** @var int    Maximalni delka logovane hodnoty: -1=neomezeno, 0=nelogovat, >0 pocet znaku */
    public $maxLogLength;

    /** @var bool   V insertu se nevyplnuje */
    public $noInsert;

    /** @var mixed  Hodnota nactena z databaze */
    public $originalValue;
    /** @var mixed  Hodnota zadana uzivatelem */
    public $value;

    public $convertTargetCharset = false;


    function __construct($name, $format, $readOnly = false, $size = 0)
    {
        $this ->fieldName = $name;
        $this ->format = $format;
        $this ->size = (int)$size;

        $this ->readOnly = $readOnly;

        $this ->updateLog = false;
        $this ->noInsert = false;

        $this ->maxLogLength = self::DEFAULT_MAX_LOG_LENGTH;
    }


    function setValue($value)
    {
        if ($this ->size > 0 && isset($value) && $value != '') {
            $value = StringUtils::substr($value, 0, $this ->size);
        }

        // Nastavi hodnotu promenne
        $this ->value = $value;

        return $this;
    }


    function setNewValue($value)
    {
        $this ->setValue($value);
        $this ->readOnly = false;
        return $this;
    }

    function setUpdateLog($value)
    {
        $this ->updateLog = $value;
        return $this;
    }

    /**
     * @param boolean $noInsert
     * @return DBRecordField
     */
    public function setNoInsert($noInsert)
    {
        $this->noInsert = $noInsert;
        return $this;
    }

    /**
     * @param boolean $readOnly
     * @return DBRecordField
     */
    public function setReadOnly($readOnly)
    {
        $this->readOnly = $readOnly;
        return $this;
    }

    /**
     * @param boolean $changeable
     * @return DBRecordField
     */
    public function setChangeable($changeable)
    {
        $this->changeable = $changeable;
        return $this;
    }

    /**
     * @param boolean $convertTargetCharset
     * @return DBRecordField
     */
    public function setConvertTargetCharset($convertTargetCharset)
    {
        $this->convertTargetCharset = $convertTargetCharset;
        return $this;
    }

    /**
     * @param string $format
     * @return DBRecordField
     */
    public function setFormat($format)
    {
        $this->format = $format;
        return $this;
    }

    /**
     * @param int $size
     * @return DBRecordField
     */
    public function setSize($size)
    {
        $this->size = $size;
        return $this;
    }

    /**
     * @param string $sequenceName
     * @return $this
     */
    public function setSequenceName($sequenceName)
    {
        $this->sequenceName = $sequenceName;
        return $this;
    }

    /**
     * @param int $maxLogLength
     * @return DBRecordField
     */
    public function setMaxLogLength($maxLogLength)
    {
        $this->maxLogLength = $maxLogLength;
        return $this;
    }


    /**
     * Vraci hodnotu pro zapis do SQL dotazu
     * @return string
     */
    function getSQLValue()
    {
        if (!isset($this ->value) || $this ->value === '') {
            return 'null';
        }

        switch ($this ->format) {
            case 'int' :
            case 'integer' :
                return (int)$this ->value;

            case 'float' :
            case 'double' :
                return (float)$this ->value;

            case 'date' :
                return $this ->record ->database ->dateToString($this ->value);

            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->timestampToString($this ->value);

            case 'sqltimestamp' : // hodnota zadana ve value je nacteny timestamp z databaze (pouziva se v XML)
                return $this ->record ->database ->timestampToString($this ->record ->database ->stringToTimestamp($this ->value));

            case 'sqldate' :
                return $this ->record ->database ->dateToString($this ->record ->database ->stringToTimestamp($this ->value));

            case 'datestr' :
                return $this ->record ->database ->timestampToString(DateUtils::dateStringToTimestamp($this ->value));

            case 'time' :
                return $this ->record ->database ->timeToString($this ->value);

            case 'memo' :
            case 'blob' :
            case 'binary' : // stejne jako blob, ale neloguje se hodnota
            default :
                return "'" . $this ->record ->database ->addSlashes($this ->value) . "'";
        }
    }


    /**
     * Vraci hodnotu pro zapis do SQL parametru
     * @return string
     */
    function getSQLParam()
    {
        $dbValue = $this ->value;

        if ($this ->convertTargetCharset  &&  $dbValue) {
            // v pripade potreby zkonvertuje znakovou sadu. Pouziva se u blob poli, do kterych se uklada text, protoze u nich neprovede konverzi driver
            global $db_target_charset;
            if ($db_target_charset) {
                $charset = TextManager::getInstance() ->getCharSet();
                if ($db_target_charset != $charset) {
                    $dbValue = iconv($charset, $db_target_charset, $dbValue);
                }
            }
        }

        if ($this ->format == 'blob'  ||  $this ->format == 'binary') {
            // pro sqlsrv je potreba i null hodnotu nebo prazdny retezec prohnat pres getBlobParam!
            return $this ->record ->database ->getBlobParam($dbValue);
        }

        if (!isset($dbValue) || $dbValue === '') {
            return null;
        }

        switch ($this ->format) {
            case 'int' :
            case 'integer' :
                return (int)$dbValue;

            case 'float' :
            case 'double' :
                return (float)$dbValue;

            case 'date' :
                return $this ->record ->database ->dateToParam($dbValue);

            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->timestampToParam($dbValue);

            case 'sqltimestamp' : // hodnota zadana ve value je nacteny timestamp z databaze (pouziva se v XML)
                return $this ->record ->database ->timestampToParam($this ->record ->database ->stringToTimestamp($dbValue));

            case 'sqldate' :
                return $this ->record ->database ->dateToParam($this ->record ->database ->stringToTimestamp($dbValue));

            case 'datestr' :
                return $this ->record ->database ->timestampToParam(DateUtils::dateStringToTimestamp($dbValue));

            case 'time' :
                return $this ->record ->database ->timeToParam($dbValue);

            case 'memo' :
            default :
                return $dbValue;
        }
    }


    /**
     * Prevede sql hodnotu na datovou hodnotu (int/float/...)
     * @param string $sqlValue
     * @return mixed
     */
    function getValueFromSQLData($sqlValue)
    {
        if (!isset($sqlValue)) {
            return null;
        }

        if ($this ->convertTargetCharset  &&  $sqlValue) {
            // v pripade potreby zkonvertuje znakovou sadu. Pouziva se u blob poli, do kterych se uklada text, protoze u nich neprovede konverzi driver
            global $db_target_charset;
            if ($db_target_charset) {
                $charset = TextManager::getInstance() ->getCharSet();
                if ($db_target_charset != $charset) {
                    $sqlValue = iconv($db_target_charset, $charset, $sqlValue);
                }
            }
        }

        switch ($this ->format) {
            case 'date' :
            case 'datetime' :
            case 'timestamp' :
                return $this ->record ->database ->stringToTimestamp($sqlValue);

            case 'time' :
                //return $this ->record ->database ->stringToTimestamp($sqlvalue);
                return $this ->record ->database ->stringToSecondsOfDay($sqlValue);

            case 'float' :
                return (float)$sqlValue;

            case 'int' :
                return (int)$sqlValue;

            case 'datestr' :
                return $sqlValue; // tohle by nemelo nastat

            case 'blob' :
            case 'binary' :
                return $this ->record ->database ->stringToBinary($sqlValue);

            //case 'sqltimestamp' : - neprevadi, zanecha puvodni DB format
            //case 'sqldate' : - neprevadi
        }

        return $sqlValue;
    }


    /**
     * Naformatuje hodnotu pro textove zobrazeni - napr. do logu
     * @param $value
     * @return bool|int|null|string
     */
    function formatValue($value)
    {
        if (!isset($value) || $value === '') {
            return '';
        }

        switch ($this ->format) {
            /** @noinspection PhpMissingBreakStatementInspection */
            case 'sqldate' :
                $value = $this ->record ->database ->stringToTimestamp($value); // value je v databazovem formatu - nejdrive se musi prevest
            // pokracuje dale!
            case 'date' :
                return date('j.n.Y', (int)$value);

            /** @noinspection PhpMissingBreakStatementInspection */
            case 'sqltimestamp' :
                $value = $this ->record ->database ->stringToTimestamp($value);
            // pokracuje dale!
            case 'datetime' :
            case 'datestr' :
            case 'timestamp' :
                return date('j.n.Y G:i:s', (int)$value);

            case 'time' :
                return DateUtils::secondsToString($value); //date('j.n.Y G:i:s', (int)$value);


            case 'binary' :
                return '(' . strlen($this ->value) . 'B' . ")";
            case 'blob' :
                return '(' . strlen($this ->value) . 'B' . ")\n" . $value;
        }

        return $value;
    }


    function isChanged()
    {
        if (!$this ->changeable  &&  isset($this ->originalValue)) {
            return false;
        }

        if ($this ->format == 'float') {
            if (isset($this ->originalValue) && !isset($this ->value)) {
                return true;
            }
            if (isset($this ->value) && !isset($this ->originalValue)) {
                return true;
            }

            if (NumberUtils::floatsEqual($this ->originalValue, $this ->value)) {
                return false;
            }

            return true;
        }

        if (isset($this ->originalValue)  &&  ($this ->originalValue !== '')  && !isset($this ->value)) {
            return true;
        }
        if (!isset($this ->originalValue)  &&  isset($this ->value)) {
            return $this ->value !== '';
        }

        if ($this ->format == 'string'  ||  $this ->format == 'memo'  ||  $this ->format == '') {
            // pro retezcova pole kontroluje presnou shodu, aby fungovala i zmena napr. z 0068 na 68
            return $this ->originalValue !== $this ->value;
        }

        return $this ->originalValue != $this ->value;
    }


    function getLogValue()
    {
        if (!$this ->maxLogLength) {
            // 0 = nelogovat
            return '';
        }

//        $value = $this ->readOnly
//            ? $this ->originalValue
//            : (($this ->changeable  ||  !isset($this ->originalValue)) ? $this ->value : $this ->originalValue);

        $value = $this ->value;

        if (!isset($value)) {
            return '(NULL)';
        }

        // TODO: umoznit logovat jen poslednich X znaku? napr. pokud bude zadano -100, tak bude logovat jen poslednich 100 znaku.
        // (muze se hodit napr. pro HistorieSchvalovani)
        $log = $this ->formatValue($value);
        if ($this ->maxLogLength > 0) {
            $log = StringUtils::substr($log, 0, $this ->maxLogLength);
        }

        return $log;
    }
}
